//=============================================================================
//
//
// Buttons_test
//
// main file
//
// This file is part of the gertboard test suite
//
//
// Copyright (C) Gert Jan van Loo & Myra VanInwegen 2012
// No rights reserved
// You may treat this program as if it was in the public domain
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
//
//
// Modified by Walter Lain 2013-2015 for use with PIOboard
//

#include "gb_common.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <assert.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <unistd.h>

// GPIO0/2=  unused
// GPIO1/3=  unused

//
// Set GPIO pins to the right mode
// button test GPIO mapping:
//           Function            Mode
// GPIO0/2=  unused
// GPIO1/3=  unused
// GPIO4=    unused
// GPIO7=    unused
// GPIO8=    unused
// GPIO9=    unused
// GPIO10=   unused
// GPIO11=   unused
// GPIO14=   unused
// GPIO15=   LED                 Input
// GPIO17=   LED                 Input
// GPIO18=   LED                 Input
// GPIO22=   LED                 Input
// GPIO23=   Pushbutton (B4)     Input
// GPIO24=   Pushbutton (B3)     Input
// GPIO25=   Pushbutton (B2)     Input
// GPIO21/27=Pushbutton (B1)     Input
//
// Always call INP_GPIO(x) first
// as that is how the macros work
void setup_gpio(int pirev)
{
   // for this test we are using GP15,17,18,22,23,24,25 & 21/27
  INP_GPIO(15);
  INP_GPIO(17);
  INP_GPIO(18);
  if(pirev==2||pirev==99)
    INP_GPIO(27);  //rasPI revision 2 has GP27
  else
    INP_GPIO(21);  //rasPI revision 1 has GP21, revert to this if for some reason the revision can't be read
  INP_GPIO(22);
  INP_GPIO(23);
  INP_GPIO(24);
  INP_GPIO(25);

  // enable pull-up on GPIO 23,24,25 & 21/27 set pull to 2 (code for pull high)
  GPIO_PULL = 2;
  short_wait();
  // setting bits 23,24,25 & 21/27 below means that the GPIO_PULL is applied to 
  // GPIO 23, 24, 25 & 21/27
  if(pirev==2||pirev==99)
    GPIO_PULLCLK0 = 0x0B800000;
  else
    GPIO_PULLCLK0 = 0x03A00000;
  short_wait();
  GPIO_PULL = 0;
  GPIO_PULLCLK0 = 0;
} // setup_gpio

// remove pulling on pins so they can be used for somnething else next time
// gertboard is used
void unpull_pins(int pirev)
{
   // to disable pull-up on GPIO 23,24,25 & 21/27, set pull to 0 (code for no pull)
   GPIO_PULL = 0;
   short_wait();
  // setting bits 23,24,25 & 21/27 below means that the GPIO_PULL is applied to 
  // GPIO 23, 24, 25 & 21/27
  if(pirev==2||pirev==99)
    GPIO_PULLCLK0 = 0x0B800000;
  else
    GPIO_PULLCLK0 = 0x03A00000;
   short_wait();
   GPIO_PULL = 0;
   GPIO_PULLCLK0 = 0;
} // unpull_pins


int main(void)
{ int r,d,pirev;
  unsigned int b,prev_b;
  char str [14];
  pirev=get_revision();
  system("clear");
  printf("\t\tPiOBoard Test suite\nBased on Gertboard test suite by G.J. van Loo, adapted by W.Lain\n\n");
  printf("<[ Buttons to LEDs test ]>\n\n");

  if(pirev==99)
    printf ("Detected Raspberry PI 2\n");
  else
    printf ("Detected Raspberry PI 1 revision [%i]\n",pirev);
  printf ("These are the connections for the test:\n");
  printf ("GP27 in J2 --- B1 in J3\n");
  printf ("GP25 in J2 --- B2 in J3\n");
  printf ("GP24 in J2 --- B3 in J3\n");
  printf ("GP23 in J2 --- B4 in J3\n");
  printf ("GP22 in J2 --- B5 in J3\n");
  printf ("GP18 in J2 --- B6 in J3\n");
  printf ("GP17 in J2 --- B7 in J3\n");
  printf ("GP15 in J2 --- B8 in J3\n");
  printf ("U3-out-B1 pin 1 (P2) --- BUF5 in J10A1\n");
  printf ("U3-out-B2 pin 1 (P4) --- BUF6 in J10A1\n");
  printf ("U3-out-B3 pin 1 (P6) --- BUF7 in J10B1\n");
  printf ("U3-out-B4 pin 1 (P8) --- BUF8 in J10B1\n");
  printf ("jumper on U4-in-B5 (P9)\n");
  printf ("jumper on U4-in-B6 (P11)\n");
  printf ("jumper on U4-in-B7 (P13)\n");
  printf ("jumper on U4-in-B8 (P15)\n\n");
  printf ("When ready hit enter.\n");
  (void) getchar();

   // Map the I/O sections
   setup_io();

   // Set GPIO pins to the required mode
   setup_gpio(pirev);

   // read the switches a number of times and print out the result

   /* below we set prev_b to a number which will definitely be different
      from what the switches returns, as after shift & mask, b can only
      be in range 0..7 */
   prev_b = 8; 
   
   r = 20; // number of repeats

  while (r)
  {
    b = GPIO_IN0;
    b = (b >> 15 ) & 0x17CD; // keep only bits 15 thru 27
    if (b^prev_b)
    { // one or more buttons changed
      make_binary_string(13, b, str);
      if(pirev==2||pirev==99)
	printf("%c%c%c%c - %c%c%c%c\n", str[0],str[2],str[3],str[4],str[5],str[9],str[10],str[12]); //shows only the used bits
      else
	printf("%c%c%c%c - %c%c%c%c\n", str[6],str[2],str[3],str[4],str[5],str[9],str[10],str[12]); //shows only the used bits
      prev_b = b;
      r--;
    } // change
  } // while

  // disable pull up on pins & unmap gpio
  unpull_pins(pirev);
  restore_io();

  return 0;
} // main

